﻿using ForumContracts.BindingModels;
using ForumContracts.SearchModels;
using ForumContracts.StorageContracts;
using ForumContracts.ViewModels;
using ForumDatabaseImplement.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ForumDatabaseImplement.Implements
{
    public class AnswerStorage : IAnswerStorage
    {
        public AnswerViewModel? Delete(AnswerBindingModel model)
        {
            using var context = new ForumDataBase();

            var element = context.Answers
                .Include(x => x.User)
                .Include(x => x.Question)
                .FirstOrDefault(rec => rec.Id == model.Id);

            if (element != null)
            {
                context.Answers.Remove(element);
                context.SaveChanges();

                return element.GetViewModel;
            }

            return null;
        }

        public AnswerViewModel? GetElement(AnswerSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }

            using var context = new ForumDataBase();

            return context.Answers
                    .Include(x => x.User)
                    .Include(x => x.Question)
                    .FirstOrDefault(x => (model.Id.HasValue && x.Id == model.Id))
                    ?.GetViewModel;
        }

        public List<AnswerViewModel> GetFilteredList(AnswerSearchModel model)
        {
            if (model.Id.HasValue)
            {
                var result = GetElement(model);
                return result != null ? new() { result } : new();
            }

            using var context = new ForumDataBase();
            IQueryable<Answer>? queryWhere = null;

            if (model.QuestionId.HasValue)
            {
                queryWhere = context.Answers.Where(x => x.QuestionId == model.QuestionId);
            }
            else
            {
                return new();
            }

            return queryWhere
                    .Include(x => x.Question)
                    .Include(x => x.User)
                    .Select(x => x.GetViewModel)
                    .ToList();
        }

        public List<AnswerViewModel> GetFullList()
        {
            using var context = new ForumDataBase();

            return context.Answers
                .Include(x => x.User)
                .Include(x => x.Question)
                .Select(x => x.GetViewModel).ToList();
        }

        public AnswerViewModel? Insert(AnswerBindingModel model)
        {
            var newAnswer = Answer.Create(model);

            if (newAnswer == null)
            {
                return null;
            }

            using var context = new ForumDataBase();

            context.Answers.Add(newAnswer);
            context.SaveChanges();

            return context.Answers
                .Include(x => x.Question)
                .Include(x => x.User)
                .FirstOrDefault(x => x.Id == newAnswer.Id)
                ?.GetViewModel;
        }

        public AnswerViewModel? Update(AnswerBindingModel model)
        {
            using var context = new ForumDataBase();

            var answer = context.Answers
                .Include(x => x.Question)
                .Include(x => x.User)
                .FirstOrDefault(x => x.Id == model.Id);

            if (answer == null)
            {
                return null;
            }

            answer.Update(model);
            context.SaveChanges();

            return answer.GetViewModel;
        }
    }
}
