using ForumBusinessLogic.BusinessLogic;
using ForumContracts.BusinessLogicContracts;
using ForumContracts.StorageContracts;
using ForumDatabaseImplement.Implements;
using Microsoft.OpenApi.Models;
using NLog.Extensions.Logging;

var builder = WebApplication.CreateBuilder(args);

builder.Logging.SetMinimumLevel(LogLevel.Trace);
builder.Logging.AddNLog("log4net.config");

// Add services to the container.
builder.Services.AddTransient<IAnswerStorage, AnswerStorage>();
builder.Services.AddTransient<IQuestionStorage, QuestionStorage>();
builder.Services.AddTransient<IUserStorage, UserStorage>();
builder.Services.AddTransient<ICategoryStorage, CategoryStorage>();

builder.Services.AddTransient<IAnswerLogic, AnswerLogic>();
builder.Services.AddTransient<IQuestionLogic, QuestionLogic>();
builder.Services.AddTransient<IUserLogic, UserLogic>();
builder.Services.AddTransient<ICategoryLogic, CategoryLogic>();

builder.Services.AddControllers();
// Learn more about configuring Swagger/OpenAPI at https://aka.ms/aspnetcore/swashbuckle
builder.Services.AddEndpointsApiExplorer();
builder.Services.AddSwaggerGen(c =>
{
    c.SwaggerDoc("v1", new OpenApiInfo
    {
        Title = "ForumRestApi",
        Version = "v1"
    });
});

var app = builder.Build();

// Configure the HTTP request pipeline.
if (app.Environment.IsDevelopment())
{
    app.UseSwagger();
    app.UseSwaggerUI(c => c.SwaggerEndpoint("/swagger/v1/swagger.json", "ForumRestApi v1"));
}

app.UseHttpsRedirection();
AppContext.SetSwitch("Npgsql.EnableLegacyTimestampBehavior", true);
app.UseAuthorization();

app.MapControllers();

app.Run();
